/**
 * Copyright (C) 2011 Ovea <dev@ovea.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ovea.tajin.framework.jmx.annotation;

import com.ovea.tajin.framework.jmx.AnnotationMetadataAssembler;
import com.ovea.tajin.framework.jmx.JmxMetadataAssembler;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * Annotate a class to be exposed by JMX
 */
@Target(ElementType.TYPE)
@Retention(RetentionPolicy.RUNTIME)
@Inherited
@Documented
public @interface JmxBean {

    /**
     * Equivalent to {@link #objectName()}}
     */
    String value() default "";

    /**
     * Complete name of JMX export. I.e. <code>java.lang:type=ClassLoading</code>. if not given,
     * The generated name will be of the form <code>package:type=classname</code>
     */
    String objectName() default "";

    String description() default "";

    /**
     * Choose which elements to expose. By default, expose only annotated methods and fields
     */
    Class<? extends JmxMetadataAssembler> assembler() default AnnotationMetadataAssembler.class;

}
